using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;

namespace HPIConverter
{
    public partial class ConvertForm : Form
    {
        private Thread m_pConvertThread = null;

        public ConvertForm()
        {
            InitializeComponent();
            CheckForIllegalCrossThreadCalls = false;
        }

        private void OnOK(object sender, EventArgs e)
        {
            if (m_pConvertThread != null && m_pConvertThread.IsAlive)
            {
                if (MessageBox.Show("Converting process is still active. Do you want to close the window anyway?", "HPI Converter", MessageBoxButtons.OKCancel, MessageBoxIcon.Question).Equals(DialogResult.OK))
                    m_pConvertThread.Abort();
                else return;
            }

            this.Close();
        }

        private void OnBrowseInput(object sender, EventArgs e)
        {
            this.folderBrowserDialog.ShowNewFolderButton = false;
            this.folderBrowserDialog.Description = "Select Input folder with Hemera Photo Objects (*.hpi):";
            if (this.folderBrowserDialog.ShowDialog().Equals(DialogResult.OK))
                this.textBoxInput.Text = this.folderBrowserDialog.SelectedPath;
        }

        private void OnBrowseOutput(object sender, EventArgs e)
        {
            this.folderBrowserDialog.ShowNewFolderButton = true;
            this.folderBrowserDialog.Description = "Select Output folder for converted JPEG or PNG files:";
            if (this.folderBrowserDialog.ShowDialog().Equals(DialogResult.OK))
                this.textBoxOutput.Text = this.folderBrowserDialog.SelectedPath;
        }

        private void OnConvertJpeg(object sender, EventArgs e)
        {
            m_pConvertThread = new Thread(new ThreadStart(ConvertJpeg));
            m_pConvertThread.Start();
        }

        private void OnConvertPng(object sender, EventArgs e)
        {
            m_pConvertThread = new Thread(new ThreadStart(ConvertPng));
            m_pConvertThread.Start();
        }

        private void ConvertJpeg()
        {
            String strInputFolder = this.textBoxInput.Text;
            if (strInputFolder.Equals(String.Empty))
            {
                MessageBox.Show("Please select valid input folder", "HPI Converter", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                m_pConvertThread = null;
                return;
            }

            DirectoryInfo directoryInputInfo = new DirectoryInfo(strInputFolder);
            if (directoryInputInfo.Exists == false)
            {
                MessageBox.Show("Please select valid input folder", "HPI Converter", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                m_pConvertThread = null;
                return;
            }

            String strOutputFolder = this.textBoxOutput.Text;
            if (strOutputFolder.Equals(String.Empty))
            {
                MessageBox.Show("Please select valid output folder", "HPI Converter", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                m_pConvertThread = null;
                return;
            }

            DirectoryInfo directoryOutputInfo = new DirectoryInfo(strOutputFolder);
            if (directoryOutputInfo.Exists == false)
            {
                MessageBox.Show("Please select valid output folder", "HPI Converter", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                m_pConvertThread = null;
                return;
            }

            FileInfo[] fileInfo = directoryInputInfo.GetFiles();

            int nLen = fileInfo.Length;

            if (nLen > 0)
            {
                int nStep = 100 / nLen;
                this.progressBar.Value = 0;

                String strExtension;
                String strNewFileName;

                foreach (FileInfo file in fileInfo)
                {
                    strExtension = file.Extension;
                    strExtension = strExtension.ToLower();
                    if (strExtension == ".hpi")
                    {
                        strNewFileName = file.Name;
                        strNewFileName = strNewFileName.Substring(0, strNewFileName.IndexOf('.'));
                        strNewFileName += ".jpg";
                        strNewFileName = directoryOutputInfo.FullName + "\\" + strNewFileName;

                        try
                        {
                            HPIHelper.ConvertToJpeg(file.FullName, strNewFileName);
                            this.progressBar.Value += nStep;
                        }
                        catch (Exception)
                        {
                            // catch anything here
                        }
                    }
                }

                this.progressBar.Value = 100;
                Thread.Sleep(500);

                this.progressBar.Value = 0;
            }

            m_pConvertThread = null;
        }

        private void ConvertPng()
        {
            String strInputFolder = this.textBoxInput.Text;
            if (strInputFolder.Equals(String.Empty))
            {
                MessageBox.Show("Please select valid input folder", "HPI Converter", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                m_pConvertThread = null;
                return;
            }

            DirectoryInfo directoryInputInfo = new DirectoryInfo(strInputFolder);
            if (directoryInputInfo.Exists == false)
            {
                MessageBox.Show("Please select valid input folder", "HPI Converter", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                m_pConvertThread = null;
                return;
            }

            String strOutputFolder = this.textBoxOutput.Text;
            if (strOutputFolder.Equals(String.Empty))
            {
                MessageBox.Show("Please select valid output folder", "HPI Converter", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                m_pConvertThread = null;
                return;
            }

            DirectoryInfo directoryOutputInfo = new DirectoryInfo(strOutputFolder);
            if (directoryOutputInfo.Exists == false)
            {
                MessageBox.Show("Please select valid output folder", "HPI Converter", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                m_pConvertThread = null;
                return;
            }

            FileInfo[] fileInfo = directoryInputInfo.GetFiles();

            int nLen = fileInfo.Length;

            if (nLen > 0)
            {
                int nStep = 100 / nLen;
                this.progressBar.Value = 0;

                String strExtension;
                String strNewFileName;

                foreach (FileInfo file in fileInfo)
                {
                    strExtension = file.Extension;
                    strExtension = strExtension.ToLower();
                    if (strExtension == ".hpi")
                    {
                        strNewFileName = file.Name;
                        strNewFileName = strNewFileName.Substring(0, strNewFileName.IndexOf('.'));
                        strNewFileName += ".png";
                        strNewFileName = directoryOutputInfo.FullName + "\\" + strNewFileName;

                        try
                        {
                            HPIHelper.ConvertToPng(file.FullName, strNewFileName);
                            this.progressBar.Value += nStep;
                        }
                        catch (Exception)
                        {
                            // catch anything here
                        }
                    }
                }

                this.progressBar.Value = 100;
                Thread.Sleep(500);

                this.progressBar.Value = 0;
            }

            m_pConvertThread = null;
        }
    }
}