//
// Written by Igor Tolmachev, IT Samples
//        mailto:support@itsamples.com
//
// Copyright (c) 2009.
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name and all copyright 
// notices remains intact. If the source code in this file is used in 
// any  commercial application then a statement along the lines of 
// "Portions copyright (c) Igor Tolmachev, 2009" must be included in 
// the startup banner, "About" box or printed documentation. An email 
// letting me know that you are using it would be nice as well. That's 
// not much to ask considering the amount of work that went into this.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability for any damage/loss of business that
// this product may cause.
//
// Expect bugs!
// 
// Please use and enjoy, and let me know of any bugs/mods/improvements 
// that you have found/implemented and I will fix/incorporate them into 
// this file. 

using System;
using System.IO;
using System.Security.Permissions;
using System.Drawing;
using System.Drawing.Imaging;

using System.Windows.Forms;

namespace HPIConverter
{
    class HPIHelper
    {
        public Image  m_JpegImage   = null;
        public Image  m_PngImage    = null;
        public Byte[] m_byJpegBytes = null;
        public Byte[] m_byPngBytes = null;
        public String m_strFileName = null;

        public bool LoadFile(String strFile)
        {
            m_strFileName = null;
            if (File.Exists(strFile))
            {
                m_strFileName = strFile;
                BinaryReader binReader = new BinaryReader(File.Open(strFile, FileMode.Open, FileAccess.Read));
                try
                {
                    Byte[] byBuffer = new Byte[4];

                    byBuffer = binReader.ReadBytes(4);
                    // 89 48 50 49  = .HPI
                    if (byBuffer[0] == 0x89 && byBuffer[1] == 0x48 && byBuffer[2] == 0x50 && byBuffer[3] == 0x49)
                    {
                        binReader.BaseStream.Position = 8;
                        int nVersion = binReader.ReadInt32();
                        if (nVersion >= 100)
                        {
                            int nJpegOffset = binReader.ReadInt32();
                            int nJpegLen = binReader.ReadInt32();
                            if (nJpegLen > 0)
                            {
                                binReader.BaseStream.Position = nJpegOffset;
                                m_byJpegBytes = binReader.ReadBytes(nJpegLen);

                                if (m_JpegImage != null) m_JpegImage.Dispose();

                                MemoryStream ms = new MemoryStream(m_byJpegBytes);

                                m_JpegImage = Image.FromStream(ms);
                                if (ms != null) ms.Dispose();
                            }

                            binReader.BaseStream.Position = 20;
                            int nPngOffset = binReader.ReadInt32();
                            int nPngLen = binReader.ReadInt32();
                            if (nPngLen > 0)
                            {
                                binReader.BaseStream.Position = nPngOffset;
                                m_byPngBytes = binReader.ReadBytes(nPngLen);

                                if (m_PngImage != null) m_PngImage.Dispose();

                                MemoryStream ms = new MemoryStream(m_byPngBytes);

                                m_PngImage = Image.FromStream(ms);
                                if (ms != null) ms.Dispose();
                            }

                            binReader.Close();

                            return true;
                        }

                    }
                }
                catch (Exception)
                {
                    return false;
                }
                finally
                {
                    binReader.Close();
                }
            }

            return false;
        }

        public Image GetImage()
        {
           if (m_JpegImage != null) return m_JpegImage;
           else if (m_PngImage != null) return m_PngImage;
           
           return null;
        }
        
        public Image GetJpegImage()
        {
            if (m_JpegImage != null) return m_JpegImage;

            return null;
        }

        public Image GetPngImage()
        {
            if (m_JpegImage != null) return m_JpegImage;

            return null;
        }

        public Byte[] GetJpegBytes()
        {
            if (m_byJpegBytes != null) return m_byJpegBytes;

            return null;
        }

        public Byte[] GetPngBytes()
        {
            if (m_byPngBytes != null) return m_byPngBytes;

            return null;
        }

        public String GetFileName()
        {
            String strFileName = m_strFileName;
            strFileName = strFileName.Substring(strFileName.IndexOf("\\")+1);
            strFileName = strFileName.Replace(".", "\0");

            return strFileName;
        }

        public static bool ConvertToJpeg(String strInFile, String strOutFile)
        {
            if (File.Exists(strInFile))
            {
                BinaryReader binReader = new BinaryReader(File.Open(strInFile, FileMode.Open, FileAccess.Read));
                BinaryWriter binWriter = new BinaryWriter(File.Open(strOutFile, FileMode.OpenOrCreate));
                try
                {
                    Byte[] byBuffer = new Byte[4];

                    byBuffer = binReader.ReadBytes(4);
                    // 89 48 50 49  = .HPI
                    if (byBuffer[0] == 0x89 && byBuffer[1] == 0x48 && byBuffer[2] == 0x50 && byBuffer[3] == 0x49)
                    {

                        binReader.BaseStream.Position = 8;
                        int nVersion = binReader.ReadInt32();
                        if (nVersion >= 100)
                        {
                            int nJpegOffset = binReader.ReadInt32();
                            int nJpegLen = binReader.ReadInt32();
                            if (nJpegLen > 0)
                            {
                                binReader.BaseStream.Position = nJpegOffset;
                                Byte[] byJpeg = binReader.ReadBytes(nJpegLen);

                                binWriter.Write(byJpeg);

                                binWriter.Close();
                                binReader.Close();
                                return true;
                            }
                        }
                    }
                }
                catch (Exception)
                {
                    return false;
                }
                finally
                {
                    binReader.Close();
                    binWriter.Close();
                }

            }
        
            return false;
        }
        
        public static bool ConvertToPng(String strInFile, String strOutFile)
        {
            if (File.Exists(strInFile))
            {
                BinaryReader binReader = new BinaryReader(File.Open(strInFile, FileMode.Open, FileAccess.Read));
                BinaryWriter binWriter = new BinaryWriter(File.Open(strOutFile, FileMode.OpenOrCreate));
                try
                {
                    Byte[] byBuffer = new Byte[4];

                    byBuffer = binReader.ReadBytes(4);
                    // 89 48 50 49  = .HPI
                    if (byBuffer[0] == 0x89 && byBuffer[1] == 0x48 && byBuffer[2] == 0x50 && byBuffer[3] == 0x49)
                    {
                        binReader.BaseStream.Position = 8;
                        int nVersion = binReader.ReadInt32();
                        if (nVersion >= 100)
                        {
                            binReader.BaseStream.Position = 20;
                            
                            int nPngOffset = binReader.ReadInt32();
                            int nPngLen = binReader.ReadInt32();
                            if (nPngLen > 0)
                            {
                                binReader.BaseStream.Position = nPngOffset;
                                Byte[] byPng = binReader.ReadBytes(nPngLen);

                                binWriter.Write(byPng);

                                binWriter.Close();
                                binReader.Close();
                                return true;
                            }
                        }
                    }
                }
                catch (Exception)
                {
                    return false;
                }
                finally
                {
                    binReader.Close();
                    binWriter.Close();
                }

            }

            return false;
        }
    }
}
